@@ -1,13 +1,4 @@
-"""
---------------------------------------------------------------------------
-Description
---------------------------------------------------------------------------
-adripy is a set of python tools for manipulating MSC Adams Drill files
---------------------------------------------------------------------------
-Author
---------------------------------------------------------------------------
-Ben Thornton (ben.thornton@mscsofware.com)
-Simulation Consultant - MSC Software
+"""a set of python tools for manipulating MSC Adams Drill files
 """
 from os import environ
 from os import remove
@@ -31,25 +22,27 @@ env = jinja2.Environment(
 
 # Test that the user config file exists
 if 'ADRILL_USER_CFG' not in os.environ:
-    raise EnvironmentError('ADRILL_USER_CFG environment variable is not set!')
+    # raise EnvironmentError('ADRILL_USER_CFG environment variable is not set!')
+    environ['ADRILL_USER_CFG'] = os.path.join(os.environ['USERPROFILE'], '.adrill.cfg')
 elif not os.path.exists(os.environ['ADRILL_USER_CFG']):
     raise FileExistsError('The configuration file {} does not exist!  You must set the ADRILL_USER_CFG environment variable to an existing cfg file before importing adripy.'.format(os.environ['ADRILL_USER_CFG']))
 
 # Test that the user shared file exists
-if 'ADRILL_SHARED_CFG' not in os.environ:
-    raise EnvironmentError('ADRILL_SHARED_CFG environment variable is not set!')
+if 'ADRILL_SHARED_CFG' not in os.environ:    
+    if 'XDG_CACHE_HOME' in os.environ:
+        environ['ADRILL_SHARED_CFG'] = os.path.join(os.environ['XDG_CACHE_HOME'], 'adrill', 'adrill.cfg')
+    else:
+        raise EnvironmentError('ADRILL_SHARED_CFG environment variable is not set!')
 elif not os.path.exists(os.environ['ADRILL_SHARED_CFG']):
     raise FileExistsError('The configuration file {} does not exist!  You must set the ADRILL_SHARED_CFG environment variable to an existing cfg file before importing adripy.'.format(os.environ['ADRILL_SHARED_CFG']))
 
 # Test that the adams launch file exists
 if 'ADAMS_LAUNCH_COMMAND' not in os.environ:
-    raise EnvironmentError('ADAMS_LAUNCH_COMMAND environment variable is not set!')
+    # raise EnvironmentError('ADAMS_LAUNCH_COMMAND environment variable is not set!')
+    environ['ADAMS_LAUNCH_COMMAND'] = os.path.join(os.environ['XDG_CACHE_HOME'], 'common', 'mdi.bat')
 elif not os.path.exists(os.environ['ADAMS_LAUNCH_COMMAND']):
     raise FileExistsError('The adams launch file {} does not exist!  You must set the ADRILL_SHARED_CFG environment variable to an existing cfg file before importing adripy.'.format(os.environ['ADAMS_LAUNCH_COMMAND']))
 
-
-environ['USERPROFILE'] = environ['USERPROFILE']
-
 # Dictionary of TO tool length parameters
 TO_LENGTH_PARAM = {}
 TO_LENGTH_PARAM['accelerator'] = ['Accelerator_Length']
@@ -76,25 +69,23 @@ TO_LENGTH_PARAM['short_collar'] = ['Collar_Length']
 TO_LENGTH_PARAM['stabilizer'] = ['Stabilizer_Length']
 
 def turn_measure_on(string_file, tool_types=[], tool_numbers=[], tool_names=[]):
-    """
-    Modify a string file to turn measure on for the designated tools.  Tools may be designated by type, number (stack order), or name
+    """Modify a string file to turn measure on for teh designated tools.  Tools may be designated by type, number (stack order), or name.
     
     Parameters
     ----------
-    string_file :      Full path to an MSC Adams Drill string file 
-                       including the .str extension
+    string_file : str
+        Full path to an MSC Adams Drill string file (include the .str extension)
+    tool_types : list, optional
+        List of tool types as seen in the string file (e.g. pdc_bit, motor, stabilizer)
+    tool_numbers : list, optional
+        List of stack orders corresponding to tools in the string
+    tool_names : list, optional
+        List of tool names
     
-    tool_types :       List of tool type as seen in the string file
-                       (e.g. pdc_bit, motor, stabilizer)
-    
-    tool_numbers:      List of stack orders corresponding to tools in a string
-    
-    tool_names:        List of tool names.
-
-                  
     Returns
     -------
-    n                  number of tools measured
+    int
+        number of tools measured
     """
     n = 0
     mark = False
@@ -134,32 +125,35 @@ def turn_measure_on(string_file, tool_types=[], tool_numbers=[], tool_names=[]):
     rename(string_file.replace('.str','.tmp'), string_file)
     return n
 
-
 def get_tool_name(string_file, tool_type, n=1, return_full_path=True):
-    """
-    Return the name and file name of the nth tool of type 'tool_type'
-    in 'string_file'.
+    """Returns the name, filename, stack order, and group name of the `n`th tool of type `tool_type` in `string_file`.
     
     Parameters
     ----------
-    string_file :      Full path to an MSC Adams Drill string file 
-                       including the .str extension
+    string_file : str
+        Full path to an Adams drill string file (include the .str extension)
+    tool_type : str
+        Tool type as seen in the string file (e.g. pdc_bit, motor, stabilizer)
+    n : int, optional
+        If the string file has multiple tools of type `tool_type`, returns the `n`th tool of that type. (the default is 1)
+    return_full_path :  bool, optional
+        If true, returns the full path to the tool file instead of using the Adams Drill database (cdb)  (the default is True)
     
-    tool_type :        Tool type as seen in the string file
-                       (e.g. pdc_bit, motor, stabilizer)
+    Raises
+    ------
+    ValueError
+        Raised if a tool of type `tool_type` is not found in `string_file`
     
-    n :                If the string file has multiple tools of the requested
-                       tool type, use this parameter to return the nth tool.
-                       Default  is n=1.
-    return_full_path : If true, removes the Adrill cdb name and replaces with
-                       the adrill cdb file path.  Default is True
-                  
     Returns
     -------
-    tool_name  :  Name of the requested tool
-    tool_file  :  Full file path the the requested tool's property file
-    tool_stack :  Stack order of tool
-    group_name :  Tools group name if it has one
+    str
+        Name of the requested tool
+    str
+        Full filepath the the requested tool's property file
+    int
+        Stack order of tool
+    str
+        Tool's group name if it has one
     """
     tool_found = False
     fid = open(string_file,'r')
@@ -209,22 +203,22 @@ def get_toolFilename_fullNotation(toolFilename_cdbNotation):
         toolFilename_fullNotation = toolFilename_cdbNotation
     return toolFilename_fullNotation
 
-
 def get_adrill_cdbs(adrill_user_cfg, adrill_shared_cfg=None):
-    """
-    Return the names and locations of all user defined MSC Adams Drill
-    configuration databases (cdbs)
+    """Return the names and locatinos of all user defined MSC Adams Drill databases (cdbs)
     
     Parameters
     ----------
-    adrill_user_cfg : Full path to an MSC Adams Drill user configuration
-                      file.  This should be in the HOME directory
-                  
+    adrill_user_cfg : str
+        Full path to an Adams Drill user configuration file.  This hould be in the users HOME directory.
+    adrill_shared_cfg : str, optional
+        Full path to an Adams Drill shared configuration file.  This should be in the Adams Drill installation directory.  (the default is None, which means that only user cdbs will be returned.)
+        
     Returns
     -------
-    cdbs :  A dictionary with the cdb names as keys and cdb 
-            locations as values.
+    dict
+        A dictionary in which the cdb names are keys and the cdb locations are values.
     """
+
     cdbs = {}
     with open(adrill_user_cfg,'r') as fid:
         for line in fid:
@@ -249,17 +243,19 @@ def get_adrill_cdbs(adrill_user_cfg, adrill_shared_cfg=None):
     return cdbs
 
 def get_TO_param(filename, requested_parameter):
-    """
-    Return the value of a parameter in a tiem orbit file
+    """Return the value of a parameter in a tiem orbit file
     
     Parameters
     ----------
-    filename            :    Full path to a tiem orbit file
-    requested_parameter :   Name of a parameter in TO_file  
+    filename : str
+        Full path to a tiem orbit file
+    requested_parameter : str
+        Name of a parameter in TO_file  
                   
     Returns
     -------
-    requested_value :  The value assigned to TO_param in TO_file
+    str or int or float
+        The value assigned to TO_param in TO_file
     """
     # Check if CDB notation used and Convert
     filename = get_toolFilename_fullNotation(filename)
@@ -303,17 +299,19 @@ def get_TO_param(filename, requested_parameter):
 
 
 def has_tool(string_file, tool_type):
-    """
-    Returns true if string_file has at least one tool of type tool_type
+    """Returns true if string_file has at least one tool of type tool_type
     
     Parameters
     ----------
-    string_file : Full path to an Adams Drill string file
-    tool_type :    Adams Drill tool type
+    string_file : str
+        Full path to an Adams Drill string file
+    tool_type : str
+        Adams Drill tool type
                   
     Returns
     -------
-    tool_type_found: True if string_file contains at least one tool of type tool_type
+    bool
+        True if string_file contains at least one tool of type tool_type
     """
     tool_type_found = False
     fid = open(string_file,'r')
@@ -330,13 +328,14 @@ def fullNotation_to_cdbNotation(string_file):
     
     Parameters
     ----------
-    string_file :       Full path to an Adams Drill string file
+    string_file : str
+        Full path to an Adams Drill string file
                   
     Returns
     -------
-    n: Num
+    int
+        Number of replacements made
     """
-
     cdbs = get_adrill_cdbs(os.environ['ADRILL_USER_CFG'], os.environ['ADRILL_SHARED_CFG'])
     n = 0 
 
@@ -371,16 +370,17 @@ def fullNotation_to_cdbNotation(string_file):
     return n
 
 def cdbNotation_to_fullNotation(string_file):
-    """
-    Replaces all references in a string file that use CDB notation to use full path notation
+    """Replaces all references in a string file that use CDB notation to use full path notation
     
     Parameters
     ----------
-    string_file :       Full path to an Adams Drill string file
+    string_file : str
+        Full path to an Adams Drill string file
                   
     Returns
     -------
-    n: Number of replacements that were made.
+    int
+        Number of replacements made
     """
 
     cdbs = get_adrill_cdbs(os.environ['ADRILL_USER_CFG'], os.environ['ADRILL_SHARED_CFG'])
@@ -407,18 +407,19 @@ def cdbNotation_to_fullNotation(string_file):
     return n
 
 def get_cdb_path(full_filepath):    
-    """
-    Given the full path to a file located in a cdb, get_cdb_path returns the path to a
+    """Given the full path to a file located in a cdb, get_cdb_path returns the path to a
     file with the cdb path replaced by the cdb alias.  full_filepath will be returned if
     no cdb is found in the path.
 
     Parameters
     ----------
-    full_filepath :     Full file path to a file in a cdb
+    full_filepath : str
+        Full file path to a file in a cdb
                   
     Returns
     -------
-    cdb_filepath :      Path to a file with the cdb path replaced by the cdb alias.
+    str
+        Path to a file with the cdb path replaced by the cdb alias.
     """
     cdb_filepath = full_filepath
     cdbs = get_adrill_cdbs(os.environ['ADRILL_USER_CFG'], os.environ['ADRILL_SHARED_CFG'])
@@ -432,19 +433,19 @@ def get_cdb_path(full_filepath):
     return cdb_filepath
 
 def get_full_path(cdb_filepath):    
-    """
-    Given the cdb path to a file located in a cdb, get_full_path returns the path to a
+    """Given the cdb path to a file located in a cdb, get_full_path returns the path to a
     file with the cdb alias replaced by the cdb location.
 
     Parameters
     ----------
-    cdb_filepath :     Full file path to a file in a cdb
+    cdb_filepath : str
+        Full file path to a file in a cdb
                   
     Returns
     -------
-    full_filepath :      Path to a file with the cdb path replaced by the cdb alias.
+    str
+        Path to a file with the cdb path replaced by the cdb alias.
     """    
-
     # Find a string that looks like a database alias
     match = re.search('^<.+>', cdb_filepath)
     
@@ -467,36 +468,43 @@ def get_full_path(cdb_filepath):
     return full_filepath
 
 def get_cdb_location(cdb_name):
-    """
-    Returns the location of cdb 'cdb_name'
+    """Returns the location of cdb 'cdb_name'
 
     Parameters
     ----------
-    cdb_name :      Alias of a cdb
+    cdb_name : str
+        Alias of a cdb
                   
     Returns
     -------
-    cdb_location :  Location of cdb
+    str
+        Location of cdb
     """
     cdbs = get_adrill_cdbs(os.environ['ADRILL_USER_CFG'], os.environ['ADRILL_SHARED_CFG'])
     return cdbs[cdb_name]
 
 def replace_tool(string_file, old_tool_file, new_tool_file, old_tool_name='', new_tool_name='', N=0):
-    """
-    Swaps old_tool_file for new_tool_file in string_file.  Also replaces the tools Name field.
+    """Swaps old_tool_file for new_tool_file in string_file.  Also replaces the tools Name field.
     
     Parameters
     ----------
-    string_file :       Full path to an Adams Drill string file
-    old_tool_file :     Path to an Adams Drill tool property file that exists in string_file. May use full path or Adrill CDB notation.
-    new_tool_file :     Path to an Adams Drill tool property file to replace old_tool_file. May use full path or Adrill CDB notation.
-    N :                 Number of replacements to make. Default is 0 which will replace all instances.
-    old_tool_name :     Name of the tool to replace. Default is the filename.
-    new_tool_name :     Name of the new tool.  Default is the filename.
+    string_file : str
+        Full path to an Adams Drill string file
+    old_tool_file : str
+        Path to an Adams Drill tool property file that exists in string_file. May use full path or Adrill CDB notation.
+    new_tool_file : str
+        Path to an Adams Drill tool property file to replace old_tool_file. May use full path or Adrill CDB notation.
+    N : int
+        Number of replacements to make. Default is 0 which will replace all instances.
+    old_tool_name : str
+        Name of the tool to replace. Default is the filename.
+    new_tool_name : str
+        Name of the new tool.  Default is the filename.
                   
     Returns
     -------
-    n: Number of replacements that were made.
+    int
+        Number of replacements that were made
     """
     old_tool_file = old_tool_file.replace('\\','/')
     new_tool_file = new_tool_file.replace('\\','/')
@@ -605,17 +613,17 @@ def replace_tool(string_file, old_tool_file, new_tool_file, old_tool_name='', ne
     return n
 
 def get_string_length(string_file):
-    """
-    Gets the total length of the drill string defined in string_file
+    """Gets the total length of the drill string defined in `string_file`
     
     Parameters
     ----------
-    string_file :       Full path to an Adams Drill string file
+    string_file : str
+        Full path to an Adams Drill string file
                       
     Returns
     -------
-    string_length:      Cumulative length of the string
-    
+    string_length : str
+        Cumulative length of the string    
     """
     cdbs = get_adrill_cdbs(os.environ['ADRILL_USER_CFG'], os.environ['ADRILL_SHARED_CFG'])
     # print(cdbs)
@@ -656,16 +664,17 @@ def get_string_length(string_file):
     return string_length
 
 def get_number_of_tools(string_file):
-    """
-    Gets the total number of tools in a string.  Tools for which quantitiiy can be defined are only counted once.  The top drive is not included
+    """Gets the total number of tools in a string.  Tools for which quantitiiy can be defined are only counted once.  The top drive is not included
 
     Parameters
     ----------
-    string_file :       Full path to an Adams Drill string file
+    string_file : str
+        Full path to an Adams Drill string file
                       
     Returns
     -------
-    num:                Number of tools  
+    int
+        Number of tools  
     """
     with open(string_file, 'r') as fid:
         for line in fid:
@@ -675,17 +684,17 @@ def get_number_of_tools(string_file):
     return num
 
 def get_bha_length(string_file):
-    """
-    Gets the total length of the drill string defined in string_file NOT including the equivalent upper string and highest most physical string
+    """Gets the total length of the drill string defined in string_file NOT including the equivalent upper string and highest most physical string
     
     Parameters
     ----------
-    string_file :       Full path to an Adams Drill string file
-                      
+    string_file : str
+        Full path to an Adams Drill string file
+
     Returns
     -------
-    string_length:      Cumulative length of the string
-    
+    float
+        Cumulative length of the string    
     """
     cdbs = get_adrill_cdbs(os.environ['ADRILL_USER_CFG'], os.environ['ADRILL_SHARED_CFG'])
     # print(cdbs)
@@ -725,16 +734,23 @@ def get_bha_length(string_file):
     return string_length
     
 def add_cdb_to_cfg(name, loc, cfg_file):
-    """Adds cdb of name 'name' and path 'loc' to cfg_file
+    """Adds cdb of name `name` and path `loc` to `cfg_file`
     
-    Arguments:
-        name {string} -- name of cdb (e.g. example_database)
-        loc {string} -- path to cdb (e.g. C:\\example_database.cdb)
-        cfg_file {string} -- Full filename of an adams drill configuration file
+    Parameters
+    ----------
+    name : str
+        Name of cdb (e.g. example_database)
+    loc : str
+        path to cdb (e.g. C:\\example_database.cdb)
+    cfg_file : str
+        Full filename of an adams drill configuration file
     
-    Raises:
-        ValueError -- Raised if a cdb of the given name or path already exists in the given config file
-        PermissionError -- Raised if the user does not have permissiosn to edit the given config file
+    Raises
+    ------
+    ValueError
+        Raised if a cdb of the given name or path already exists in the given config file
+    PermissionError
+        Raised if the user does not have permissiosn to edit the given config file
     """
 
     loc = os.path.normpath(loc)
@@ -778,19 +794,21 @@ def add_cdb_to_cfg(name, loc, cfg_file):
         raise PermissionError('You do not have permission to edit {}.'.format(cfg_file))    
 
 def remove_cdb_from_cfg(name, cfg_file):
-    """Removes cdb of name 'name' from cfg_file
+    """Removes cdb of name `name` from `cfg_file`
     
-    ==========
-    Arguments:
-    ==========
-        name {string} -- name of cdb (e.g. example_database)
-        cfg_file {string} -- Full filename of an adams drill configuration file
+    Parameters
+    ----------
+    name : str
+        Name of cdb (e.g. example_database)
+    cfg_file : str
+        Full filename of an adams drill configuration file
     
-    ==========
-    Raises:
-    ==========
-        ValueError -- Raised if a cdb of the given name or path already exists in the given config file
-        PermissionError -- Raised if the user does not have permissiosn to edit the given config file
+    Raises
+    ------
+    ValueError
+        Raised if a cdb of the given name or path already exists in the given config file
+    PermissionError
+        Raised if the user does not have permissiosn to edit the given config file
     """
 
     # Initialize cdbs dictionary
@@ -829,12 +847,14 @@ def remove_cdb_from_cfg(name, cfg_file):
         raise PermissionError('You do not have permission to edit {}.'.format(cfg_file))  
 
 def create_cfg_file(filename, database_paths):
-    """Create a cfg file with the databases whose paths are given in the database_paths list.
-    Also sets the ADRILL_USER_CONFIG environment variable equal to filename.
+    """Create a cfg file with the databases whose paths are given in the database_paths list. Also sets the ADRILL_USER_CONFIG environment variable equal to filename.
     
-    Arguments:
-        filename {string} -- Filename for the new configuration file.
-        database_paths {list} -- List of database paths to include in the configuration file. 
+    Parameters
+    ----------
+    filename : str
+        Filename for the new configuration file.
+    database_paths : list
+        List of database paths to include in the configuration file. 
     """
 
 
@@ -856,13 +876,16 @@ def create_cfg_file(filename, database_paths):
 def build(string_file, solver_settings_file, working_directory, output_name=None):    
     """Builds adm, acf, and cmd files from string, event, and solver settings files.
     
-    Arguments:
-        string {str} -- Filename of a drill string (.str) file.
-        solver_settings {str} -- Filename of a solver settings (.ssf) file.
-        working_directory {string} -- Path to the directory to put the adm, acf, and cmd.
-    
-    Keyword Arguments:
-        output_name {string} -- Base name of the adm, acf, and cmd files. (default: Same as string_file)
+    Parameters
+    ----------
+    string : str
+        Filename of a drill string (.str) file.
+    solver_settings : str
+        Filename of a solver settings (.ssf) file.
+    working_directory : str
+        Path to the directory to put the adm, acf, and cmd
+    output_name : str
+        Base name of the adm, acf, and cmd files. (the default is none, which redefines the `ouput_name` to be the same as the 'OutputName' parameter in the string file)
     """    
     # Set the output name  
     if output_name is None:  
