@@ -1,12 +1,15 @@
+"""Tests related to the tiem_orbit package
+"""
+
 import unittest
 import os
-import glob
 from test import *
 os.environ['ADRILL_SHARED_CFG'] = os.path.join('C:\\', 'MSC.Software', 'Adams', '2018', 'adrill', 'adrill.cfg')
 os.environ['ADRILL_USER_CFG'] = os.path.join(os.environ['USERPROFILE'], '.adrill.cfg')
 os.environ['ADAMS_LAUNCH_COMMAND'] = os.path.join('C:\\', 'MSC.Software', 'Adams', '2018', 'common', 'mdi.bat')
-from adamspy import adripy
-from adamspy.adripy.tiem_orbit import utilities
+    
+from adamspy import adripy #pylint: disable=wrong-import-position
+from adamspy.adripy.tiem_orbit import utilities #pylint: disable=wrong-import-position
 
 class Test_EventFile(unittest.TestCase):
     """
@@ -138,6 +141,16 @@ class Test_DrillTool(unittest.TestCase):
         expected_filename = os.path.join(f'<{TEST_DATABASE_NAME}>', 'stabilizers.tbl', self.NEW_STABILIZER_NAME + '.sta')
         self.assertEqual(self.stabilizer_to_modify.property_file, expected_filename)
   
+    def test_get_parameter_value_bit_length(self):
+        """Tests that DrillTool.get_parameter_value() returns the correct bit length.
+        """
+        pdc_bit = adripy.tiem_orbit.DrillTool(TEST_PDC_FILE)
+        expected_value = 1.5
+        actual_value = pdc_bit.get_parameter_value('Bit_Length')
+
+        self.assertEqual(actual_value, expected_value)
+
+
     def test_modify_parameter_value(self):
         """Modify a parameter value then get the parameter value and test the expected value is returned.
         """
@@ -236,7 +249,7 @@ class Test_DrillString(unittest.TestCase):
         drill_string.add_tool(self.top_drive)
 
         # Publish drill string to new database
-        drill_string.write_to_file(cdb=TEST_NEW_DATABASE_NAME, publish=True)
+        drill_string.write_to_file(cdb=TEST_NEW_DATABASE_NAME, publish=True, publish_event=True)
 
         expected_string_filename = os.path.join(TEST_NEW_DATABASE_PATH, 'drill_strings.tbl', TEST_STRING_NAME + '.str')
         expected_text = EXPECTED_STRING_WRITE_TEXT.replace(TEST_DATABASE_NAME, TEST_NEW_DATABASE_NAME)
@@ -423,6 +436,92 @@ class Test_DrillString(unittest.TestCase):
         # Create a DrillString object
         drill_string = adripy.tiem_orbit.DrillString(TEST_STRING_NAME, TEST_HOLE_FILE, TEST_EVENT_FILE)
 
+        # Define a new stabilizer object
+        different_stabilizer  = adripy.tiem_orbit.DrillTool(TEST_STABILIZER_FILE)
+
+        # Add the DrillTool objects to the DrillString object
+        drill_string.add_tool(self.pdc_bit, measure='yes')
+        drill_string.add_tool(self.stabilizer, measure='yes')
+        drill_string.add_tool(different_stabilizer, measure='no')
+        drill_string.add_tool(self.drill_pipe, joints=20, group_name='Upper_DP_Group')
+        drill_string.add_tool(self.eus, joints=20, group_name='equivalent_pipe', equivalent=True)
+        drill_string.add_tool(self.top_drive)
+
+        got_tool = drill_string.get_tool('stabilizer')
+
+        self.assertEqual(got_tool, self.stabilizer)    
+
+    def test_get_tool_last(self):
+        """Tests that DrillString.get_tool() returns the correct value.
+        """
+        # Create a DrillString object
+        drill_string = adripy.tiem_orbit.DrillString(TEST_STRING_NAME, TEST_HOLE_FILE, TEST_EVENT_FILE)
+        
+        # Define a new stabilizer object
+        different_stabilizer  = adripy.tiem_orbit.DrillTool(TEST_STABILIZER_FILE)
+        
+        # Add the DrillTool objects to the DrillString object
+        drill_string.add_tool(self.pdc_bit, measure='yes')
+        drill_string.add_tool(self.stabilizer, measure='yes')
+        drill_string.add_tool(different_stabilizer, measure='no')
+        drill_string.add_tool(self.drill_pipe, joints=20, group_name='Upper_DP_Group')
+        drill_string.add_tool(self.eus, joints=20, group_name='equivalent_pipe', equivalent=True)
+        drill_string.add_tool(self.top_drive)
+
+        got_tool = drill_string.get_tool('stabilizer', instance='last')
+
+        self.assertEqual(got_tool, different_stabilizer)  
+
+    def test_get_tool_1(self):
+        """Tests that DrillString.get_tool() returns the correct value.
+        """
+        # Create a DrillString object
+        drill_string = adripy.tiem_orbit.DrillString(TEST_STRING_NAME, TEST_HOLE_FILE, TEST_EVENT_FILE)
+        
+        # Define a new stabilizer object
+        different_stabilizer  = adripy.tiem_orbit.DrillTool(TEST_STABILIZER_FILE)
+        
+        # Add the DrillTool objects to the DrillString object
+        drill_string.add_tool(self.pdc_bit, measure='yes')
+        drill_string.add_tool(self.stabilizer, measure='yes')
+        drill_string.add_tool(different_stabilizer, measure='no')
+        drill_string.add_tool(self.drill_pipe, joints=20, group_name='Upper_DP_Group')
+        drill_string.add_tool(self.eus, joints=20, group_name='equivalent_pipe', equivalent=True)
+        drill_string.add_tool(self.top_drive)
+
+        got_tool = drill_string.get_tool('stabilizer', instance=1)
+
+        self.assertEqual(got_tool, self.stabilizer)  
+
+    def test_get_tool_2(self):
+        """Tests that DrillString.get_tool() returns the correct value.
+        """
+        # Create a DrillString object
+        drill_string = adripy.tiem_orbit.DrillString(TEST_STRING_NAME, TEST_HOLE_FILE, TEST_EVENT_FILE)
+        
+        # Define a new stabilizer object
+        different_stabilizer  = adripy.tiem_orbit.DrillTool(TEST_STABILIZER_FILE)
+        
+        # Add the DrillTool objects to the DrillString object
+        drill_string.add_tool(self.pdc_bit, measure='yes')
+        drill_string.add_tool(self.stabilizer, measure='yes')
+        drill_string.add_tool(different_stabilizer, measure='no')
+        drill_string.add_tool(self.drill_pipe, joints=20, group_name='Upper_DP_Group')
+        drill_string.add_tool(self.eus, joints=20, group_name='equivalent_pipe', equivalent=True)
+        drill_string.add_tool(self.top_drive)
+
+        got_tool = drill_string.get_tool('stabilizer', instance=2)
+
+        self.assertEqual(got_tool, different_stabilizer)  
+
+    def test_set_pipe_joints_physical(self):
+        """Tests that DrillString.set_pipe_joints() correctly 
+        sets the number of joints of physical pipe.
+        """
+        expected_joints = 100
+        # Create a DrillString object
+        drill_string = adripy.tiem_orbit.DrillString(TEST_STRING_NAME, TEST_HOLE_FILE, TEST_EVENT_FILE)
+
         # Add the DrillTool objects to the DrillString object
         drill_string.add_tool(self.pdc_bit, measure='yes')
         drill_string.add_tool(self.stabilizer, measure='yes')
@@ -431,10 +530,53 @@ class Test_DrillString(unittest.TestCase):
         drill_string.add_tool(self.eus, joints=20, group_name='equivalent_pipe', equivalent=True)
         drill_string.add_tool(self.top_drive)
 
-        got_tool = drill_string.get_tool('stabilizer')
+        drill_string.set_pipe_joints(expected_joints)
 
-        self.assertEqual(got_tool, self.stabilizer)      
+        actual_joints = drill_string.tools[-2]['Number_of_Joints']
 
+        self.assertEqual(actual_joints, expected_joints)
+
+    def test_set_pipe_joints_equivalent(self):
+        """Tests that DrillString.set_pipe_joints() correctly 
+        sets the number of joints of equivalent pipe.
+        """
+        expected_joints = 100
+        # Create a DrillString object
+        drill_string = adripy.tiem_orbit.DrillString(TEST_STRING_NAME, TEST_HOLE_FILE, TEST_EVENT_FILE)
+
+        # Add the DrillTool objects to the DrillString object
+        drill_string.add_tool(self.pdc_bit, measure='yes')
+        drill_string.add_tool(self.stabilizer, measure='yes')
+        drill_string.add_tool(self.stabilizer, measure='no')
+        drill_string.add_tool(self.drill_pipe, joints=20, group_name='Upper_DP_Group')
+        drill_string.add_tool(self.eus, joints=20, group_name='equivalent_pipe', equivalent=True)
+        drill_string.add_tool(self.top_drive)
+
+        drill_string.set_pipe_joints(expected_joints, equivalent=True)
+
+        actual_joints = drill_string.tools[-1]['Number_of_Joints']
+
+        self.assertEqual(actual_joints, expected_joints)
+
+    def test_get_bha_length(self):
+        """Tests that DrillString.get_bha_length() returns the correct length.
+        """
+        # Create a DrillString object
+        drill_string = adripy.tiem_orbit.DrillString(TEST_STRING_NAME, TEST_HOLE_FILE, TEST_EVENT_FILE)
+
+        # Add the DrillTool objects to the DrillString object
+        drill_string.add_tool(self.pdc_bit, measure='yes')
+        drill_string.add_tool(self.stabilizer, measure='yes')
+        drill_string.add_tool(self.stabilizer, measure='no')
+        drill_string.add_tool(self.drill_pipe, joints=20, group_name='Upper_DP_Group')
+        drill_string.add_tool(self.eus, joints=20, group_name='equivalent_pipe', equivalent=True)
+        drill_string.add_tool(self.top_drive)
+
+        expected_length = 14.48
+        actual_length = drill_string.get_bha_length()
+
+        self.assertEqual(actual_length, expected_length)
+        
     def tearDown(self):
         # Delete test config file
         os.remove(TEST_CONFIG_FILENAME)
@@ -450,7 +592,9 @@ class Test_DrillString(unittest.TestCase):
         clear_database(TEST_NEW_DATABASE_PATH)
 
 class Test_DrillSolverSettings(unittest.TestCase):
-
+    
+    maxDiff = None
+    
     def setUp(self):        
         # Create a configuration file for testing
         adripy.create_cfg_file(TEST_CONFIG_FILENAME, [TEST_DATABASE_PATH])
@@ -465,6 +609,8 @@ class Test_DrillSolverSettings(unittest.TestCase):
         solver_settings_from_file = adripy.tiem_orbit.DrillSolverSettings.read_from_file(solver_settings_file)
         
         params = dict(solver_settings_from_file.parameters)
+        params.pop('_Funnel')
+        TEST_EXPECTED_SOLVER_SETTINGS_TO_PARAMETERS.pop('_Funnel')
 
         self.assertDictEqual(params, TEST_EXPECTED_SOLVER_SETTINGS_TO_PARAMETERS)
 
@@ -473,7 +619,6 @@ class Test_DrillSolverSettings(unittest.TestCase):
         os.remove(TEST_CONFIG_FILENAME)
         os.environ['ADRILL_USER_CFG'] = os.path.join(os.environ['USERPROFILE'], '.adrill.cfg')
 
-
 class Test_ReadTOFile(unittest.TestCase):
     """Tests adamspy.adripy.tiem_orbit.utilities.read_to_file()    
     """
@@ -499,7 +644,7 @@ class Test_ReadTOFile(unittest.TestCase):
         # Create a test config file containing the test database
         parameters = utilities.read_TO_file(adripy.get_full_path(os.path.join(f'<{TEST_DATABASE_NAME}>', 'drill_strings.tbl', TEST_EXISTING_STRING_NAME + '.str')))
 
-        self.assertDictEqual(parameters, EXPECTED_STRING_TO_PARAMETERS)
+        self.assertDictEqual(parameters, EXPECTED_STRING_TO_PARAMETERS)    
 
     def tearDown(self):
         try:
